@@ -1,57 +1,35 @@
-/* ====================================================================
- * The Apache Software License, Version 1.1
- *
- * Authors:
- *	Daniel Lopez Ridruejo
- * 	Gonzalo Paniagua Javier
- *
- * Copyright (c) 2002 Daniel Lopez Ridruejo.
- *           (c) 2002,2003 Ximian, Inc.
- *           All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by 
- *        Daniel Lopez Ridruejo (daniel@rawbyte.com) and
- *        Ximian Inc. (http://www.ximian.com)"
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The name "mod_mono" must not be used to endorse or promote products 
- *    derived from this software without prior written permission. For written
- *    permission, please contact daniel@rawbyte.com.
- *
- * 5. Products derived from this software may not be called "mod_mono",
- *    nor may "mod_mono" appear in their name, without prior written
- *    permission of Daniel Lopez Ridruejo and Ximian Inc.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL DANIEL LOPEZ RIDRUEJO OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- */
+//
+// ModMonoRequest.cs
+//
+// Authors:
+//	Daniel Lopez Ridruejo
+// 	Gonzalo Paniagua Javier
+//
+// Copyright (c) 2002 Daniel Lopez Ridruejo.
+//           (c) 2002,2003 Ximian, Inc.
+//           All rights reserved.
+// (C) Copyright 2004 Novell, Inc. (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections;
 using System.Diagnostics;
@@ -65,32 +43,24 @@ namespace Mono.ASPNET
 	enum Cmd
 	{
 		FIRST_COMMAND,
-		GET_REQUEST_LINE = 0,
-		SEND_FROM_MEMORY,
-		GET_PATH_INFO,
+		SEND_FROM_MEMORY = 0,
 		GET_SERVER_VARIABLE,
-		GET_PATH_TRANSLATED,
-		GET_SERVER_PORT,
 		SET_RESPONSE_HEADER,
-		GET_FILENAME,
-		GET_REMOTE_ADDRESS,
-		GET_LOCAL_ADDRESS,
-		GET_REMOTE_PORT,
 		GET_LOCAL_PORT,
-		GET_REMOTE_NAME,
 		FLUSH,
 		CLOSE,
 		SHOULD_CLIENT_BLOCK,
 		SETUP_CLIENT_BLOCK,
 		GET_CLIENT_BLOCK,
-		SET_STATUS_LINE,
-		SET_STATUS_CODE,
+		SET_STATUS,
 		DECLINE_REQUEST,
+		NOT_FOUND,
 		LAST_COMMAND
 	}
 
 	public class ModMonoRequest : MarshalByRefObject
 	{
+		const int MAX_STRING_SIZE = 1024 * 10;
 		BinaryReader reader;
 		BinaryWriter writer;
 		Hashtable serverVariables = new Hashtable (CaseInsensitiveHashCodeProvider.Default,
@@ -108,6 +78,7 @@ namespace Mono.ASPNET
 		bool setupClientBlockCalled;
 		Hashtable headers;
 		int clientBlock;
+		bool shutdown;
 
 		public ModMonoRequest (NetworkStream ns)
 		{
@@ -116,8 +87,20 @@ namespace Mono.ASPNET
 			GetInitialData ();
 		}
 
+		public bool ShuttingDown {
+			get { return shutdown; }
+		}
+		
 		void GetInitialData ()
 		{
+			byte cmd = reader.ReadByte ();
+			shutdown = (cmd == 0);
+			if (shutdown)
+				return;
+
+			if (cmd != 1)
+				throw new InvalidOperationException ("mod_mono and xsp have different versions.");
+
 			verb = ReadString ();
 			uri = ReadString ();
 			queryString = ReadString ();
@@ -129,6 +112,12 @@ namespace Mono.ASPNET
 				string key = ReadString ();
 				headers [key] = ReadString ();
 			}
+
+			localAddress = ReadString ();
+			serverPort = reader.ReadInt32 ();
+			remoteAddress = ReadString ();
+			remotePort = reader.ReadInt32 ();
+			remoteName = ReadString ();
 		}
 
 		void SendSimpleCommand (Cmd cmd)
@@ -137,16 +126,12 @@ namespace Mono.ASPNET
 			writer.Write (b);
 		}
 
-		void ReadEnd ()
-		{
-			byte b = reader.ReadByte ();
-			if (b != 0)
-				throw new Exception ("Protocol violation or error");
-		}
-
 		string ReadString ()
 		{
 			int size = reader.ReadInt32 ();
+			if (size < 0 || size > MAX_STRING_SIZE)
+				throw new ArgumentOutOfRangeException ("size", "Abnormal string size.");
+
 			byte [] buf = new byte [size];
 			string s;
 			if (size != 0) {
@@ -168,7 +153,11 @@ namespace Mono.ASPNET
 		public void Decline ()
 		{
 			SendSimpleCommand (Cmd.DECLINE_REQUEST);
-			ReadEnd ();
+		}
+
+		public void NotFound ()
+		{
+			SendSimpleCommand (Cmd.NOT_FOUND);
 		}
 
 		public string GetProtocol ()
@@ -186,7 +175,6 @@ namespace Mono.ASPNET
 			SendSimpleCommand (Cmd.SEND_FROM_MEMORY);
 			writer.Write (length);
 			writer.Write (data, position, length);
-			ReadEnd ();
 		}
 
 		public void SetResponseHeader (string name, string value)
@@ -194,7 +182,6 @@ namespace Mono.ASPNET
 			SendSimpleCommand (Cmd.SET_RESPONSE_HEADER);
 			WriteString (name);
 			WriteString (value);
-			ReadEnd ();
 		}
 
 		public string [] GetAllHeaders ()
@@ -226,7 +213,6 @@ namespace Mono.ASPNET
 
 			SendSimpleCommand (Cmd.GET_SERVER_VARIABLE);
 			WriteString (name);
-			ReadEnd ();
 			o = ReadString ();
 			serverVariables [name] = o;
 
@@ -238,14 +224,6 @@ namespace Mono.ASPNET
 			return uri;
 		}
 
-		public string GetFileName ()
-		{
-			// Not used!
-			SendSimpleCommand (Cmd.GET_FILENAME);
-			ReadEnd ();
-			return ReadString ();
-		}
-
 		public string GetQueryString ()
 		{
 			return queryString;
@@ -255,45 +233,21 @@ namespace Mono.ASPNET
 		// for things like self referential URLs, etc.
 		public int GetServerPort ()
 		{
-			if (serverPort != 0)
-				return serverPort;
-
-			SendSimpleCommand (Cmd.GET_SERVER_PORT);
-			ReadEnd ();
-			serverPort = reader.ReadInt32 ();
 			return serverPort;
 		}
 
 		public string GetRemoteAddress ()
 		{
-			if (remoteAddress != null)
-				return remoteAddress;
-
-			SendSimpleCommand (Cmd.GET_REMOTE_ADDRESS);
-			ReadEnd ();
-			remoteAddress = ReadString ();
 			return remoteAddress;
 		}
 
 		public string GetRemoteName ()
 		{
-			if (remoteName != null)
-				return remoteName;
-
-			SendSimpleCommand (Cmd.GET_REMOTE_NAME);
-			ReadEnd ();
-			remoteName = ReadString ();
 			return remoteName;
 		}
 
 		public string GetLocalAddress ()
 		{
-			if (localAddress != null)
-				return localAddress;
-
-			SendSimpleCommand (Cmd.GET_LOCAL_ADDRESS);
-			ReadEnd ();
-			localAddress = ReadString ();
 			return localAddress;
 		}
 
@@ -303,32 +257,23 @@ namespace Mono.ASPNET
 				return localPort;
 
 			SendSimpleCommand (Cmd.GET_LOCAL_PORT);
-			ReadEnd ();
 			localPort = reader.ReadInt32 ();
 			return localPort;
 		}
 
 		public int GetRemotePort ()
 		{
-			if (remotePort != 0)
-				return remotePort;
-
-			SendSimpleCommand (Cmd.GET_REMOTE_PORT);
-			ReadEnd ();
-			remotePort = reader.ReadInt32 ();
 			return remotePort;
 		}
 
 		public void Flush ()
 		{
 			SendSimpleCommand (Cmd.FLUSH);
-			ReadEnd ();
 		}
 
 		public void Close ()
 		{
 			SendSimpleCommand (Cmd.CLOSE);
-			ReadEnd ();
 		}
 
 		public int SetupClientBlock ()
@@ -338,7 +283,6 @@ namespace Mono.ASPNET
 
 			setupClientBlockCalled = true;
 			SendSimpleCommand (Cmd.SETUP_CLIENT_BLOCK);
-			ReadEnd ();
 			int i = reader.ReadInt32 ();
 			clientBlock = i;
 			return i;
@@ -347,7 +291,6 @@ namespace Mono.ASPNET
 		public bool ShouldClientBlock () 
 		{
 			SendSimpleCommand (Cmd.SHOULD_CLIENT_BLOCK);
-			ReadEnd ();
 			int i = reader.ReadInt32 ();
 			return (i == 0);
 		} 
@@ -359,7 +302,6 @@ namespace Mono.ASPNET
 			
 			SendSimpleCommand (Cmd.GET_CLIENT_BLOCK);
 			writer.Write (size);
-			ReadEnd ();
 			int i = reader.ReadInt32 ();
 			if (i > size)
 				throw new Exception ("Houston...");
@@ -369,12 +311,9 @@ namespace Mono.ASPNET
 
 		public void SetStatusCodeLine (int code, string status)
 		{
-			SendSimpleCommand (Cmd.SET_STATUS_CODE);
+			SendSimpleCommand (Cmd.SET_STATUS);
 			writer.Write (code);
-			ReadEnd ();
-			SendSimpleCommand (Cmd.SET_STATUS_LINE);
 			WriteString (status);
-			ReadEnd ();
 		}
 	}
 }
