@@ -1,57 +1,35 @@
-/* ====================================================================
- * The XSP Software License, Version 1.1
- *
- * Authors:
- *	Daniel Lopez Ridruejo
- * 	Gonzalo Paniagua Javier
- *
- * Copyright (c) 2002 Daniel Lopez Ridruejo.
- *           (c) 2002,2003 Ximian, Inc.
- *           All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by 
- *        Daniel Lopez Ridruejo (daniel@rawbyte.com) and
- *        Ximian Inc. (http://www.ximian.com)"
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The name "mod_mono" must not be used to endorse or promote products 
- *    derived from this software without prior written permission. For written
- *    permission, please contact daniel@rawbyte.com.
- *
- * 5. Products derived from this software may not be called "mod_mono",
- *    nor may "mod_mono" appear in their name, without prior written
- *    permission of Daniel Lopez Ridruejo and Ximian Inc.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL DANIEL LOPEZ RIDRUEJO OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- */
+//
+// ModMonoWorkerRequest.cs
+//
+// Authors:
+//	Daniel Lopez Ridruejo
+// 	Gonzalo Paniagua Javier
+//
+// Copyright (c) 2002 Daniel Lopez Ridruejo.
+//           (c) 2002,2003 Ximian, Inc.
+//           All rights reserved.
+// (C) Copyright 2004 Novell, Inc. (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Web;
 using System.Collections;
@@ -92,6 +70,15 @@ namespace Mono.ASPNET
 		{
 			request.Decline ();
 		}
+
+		public void NotFound ()
+		{
+			request.NotFound ();
+		}
+
+		public bool ShuttingDown {
+			get { return request.ShuttingDown; }
+		}
 	}
 
 	public class ModMonoWorkerRequest : MonoWorkerRequest
@@ -109,6 +96,7 @@ namespace Mono.ASPNET
 		string remoteName;
 		ModMonoRequestBroker requestBroker;
 		string[] headers;
+		int [] headersHash;
 		string[] headerValues;
 		int requestId;
 
@@ -143,7 +131,11 @@ namespace Mono.ASPNET
 			indexFiles = (string []) files.ToArray (typeof (string));
 		}
 
-		public ModMonoWorkerRequest (int requestId, ModMonoRequestBroker requestBroker, IApplicationHost appHost, string verb, string path, string queryString, string protocol, string localAddress, int serverPort, string remoteAddress, int remotePort, string remoteName, string[] headers, string[] headerValues)
+		public ModMonoWorkerRequest (int requestId, ModMonoRequestBroker requestBroker,
+					IApplicationHost appHost, string verb, string path,
+					string queryString, string protocol, string localAddress,
+					int serverPort, string remoteAddress, int remotePort,
+					string remoteName, string[] headers, string[] headerValues)
 			: base (appHost)
 		{
 			this.requestId = requestId;
@@ -161,8 +153,7 @@ namespace Mono.ASPNET
 			this.headerValues = headerValues;
 		}
 
-		public int RequestId
-		{
+		public override int RequestId {
 			get { return requestId; }
 		}
 
@@ -216,7 +207,7 @@ namespace Mono.ASPNET
 				pathInfo = "";
 				return true;
 			} else if (path [path.Length - 1] == '/') {
-				path = path + "/" + indexFiles [0];
+				path = path + indexFiles [0];
 			}
 
 			// Yes, MS only looks for the '.'. Try setting a handler
@@ -296,12 +287,12 @@ namespace Mono.ASPNET
 		public override void SendResponseFromMemory (byte [] data, int length)
 		{
 			if (data.Length > length * 2) {
-				byte[] tmpbuffer = new byte[length];
-				Array.Copy (data, tmpbuffer, length);
+				byte [] tmpbuffer = new byte [length];
+				Buffer.BlockCopy (data, 0, tmpbuffer, 0, length);
 				requestBroker.Write (requestId, tmpbuffer, 0, length);
-			}
-			else
+			} else {
 				requestBroker.Write (requestId, data, 0, length);
+			}
 		}
 
 		public override void SendStatus (int statusCode, string statusDescription)
@@ -373,11 +364,18 @@ namespace Mono.ASPNET
 			return GetRequestHeader (GetKnownRequestHeaderName (index));
 		}
 		
-		private string GetRequestHeader (string name)
+		string GetRequestHeader (string name)
 		{
-			int i = Array.IndexOf (headers, name);
-			if (i == -1) return null;
-			else return headerValues [i];			
+			IHashCodeProvider hp = CaseInsensitiveHashCodeProvider.Default;
+			if (headersHash == null) {
+				headersHash = new int [headers.Length];
+				for (int i = 0; i < headers.Length; i++) {
+					headersHash [i] = hp.GetHashCode (headers [i]);
+				}
+			}
+
+			int k = Array.IndexOf (headersHash, hp.GetHashCode (name));
+			return (k == -1) ? null : headerValues [k];
 		}
 
 		public override void SendCalculatedContentLength (int contentLength) 
@@ -390,9 +388,9 @@ namespace Mono.ASPNET
 			if (buffer == null || size <= 0)
 				return 0;
 
-			byte[] readBuffer;
+			byte [] readBuffer;
 			int nr = requestBroker.Read (requestId, size, out readBuffer);
-			Array.Copy (readBuffer, 0, buffer, 0, nr);
+			Buffer.BlockCopy (readBuffer, 0, buffer, 0, nr);
 			return nr;
 		}
 	}
