@@ -1,53 +1,36 @@
-/* ====================================================================
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 2002 Daniel Lopez Ridruejo.
- *           (c) 2002 Ximian, Inc.
- *           All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by 
- *        Daniel Lopez Ridruejo (daniel@rawbyte.com) and
- *        Ximian Inc. (http://www.ximian.com)"
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The name "mod_mono" must not be used to endorse or promote products 
- *    derived from this software without prior written permission. For written
- *    permission, please contact daniel@rawbyte.com.
- *
- * 5. Products derived from this software may not be called "mod_mono",
- *    nor may "mod_mono" appear in their name, without prior written
- *    permission of Daniel Lopez Ridruejo and Ximian Inc.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL DANIEL LOPEZ RIDRUEJO OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- */
+//
+// MonoWorkerRequest.cs
+//
+// Authors:
+//	Daniel Lopez Ridruejo
+// 	Gonzalo Paniagua Javier
+//
+// Copyright (c) 2002 Daniel Lopez Ridruejo.
+//           (c) 2002,2003 Ximian, Inc.
+//           All rights reserved.
+// (C) Copyright 2004 Novell, Inc. (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
+
 using System;
 using System.Collections;
 using System.Diagnostics;
@@ -229,7 +212,7 @@ namespace Mono.ASPNET
 				path = path.Substring (1);
 
 			int len = HostVPath.Length;
-			if (path == HostVPath || path.StartsWith (HostVPath + "/"))
+			if (path.StartsWith (HostVPath) && (path.Length == len || path [len] == '/'))
 				path = path.Substring (len + 1);
 
 			if (path.Length > 0 && path [0] == '/')
@@ -239,6 +222,7 @@ namespace Mono.ASPNET
 		}
 
 		protected abstract bool GetRequestData ();
+		public abstract int RequestId { get; }
 
 		public bool ReadRequestData ()
 		{
