@@ -5,10 +5,30 @@
 //	Gonzalo Paniagua Javier (gonzalo@ximian.com)
 //	Lluis Sanchez Gual (lluis@ximian.com)
 //
-// (C) Copyright 2004 Novell, Inc
+// (C) Copyright 2004 Novell, Inc. (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
 using System;
+using System.IO;
 using System.Net;
 using System.Net.Sockets;
 using System.Text;
@@ -24,9 +44,9 @@ namespace Mono.ASPNET
 	{
 		IPEndPoint bindAddress;
 		
-		public XSPWebSource ()
+		public XSPWebSource (IPAddress address, int port)
 		{
-			SetListenAddress (80);
+			SetListenAddress (address, port);
 		}
 		
 		public void SetListenAddress (int port)
@@ -80,6 +100,19 @@ namespace Mono.ASPNET
 		{
 			return ((XSPWorker)GetWorker (requestId)).IsConnected ();
 		}
+
+		public int GetReuseCount (int requestId)
+		{
+			XSPWorker worker = (XSPWorker) GetWorker (requestId);
+			return worker.GetReuseCount ();
+		}
+
+		public void Close (int requestId, bool keepAlive)
+		{
+			XSPWorker worker = (XSPWorker) GetWorker (requestId);
+			if (worker != null)
+				worker.Close (keepAlive);
+		}
 	}
 	
 	//
@@ -87,18 +120,23 @@ namespace Mono.ASPNET
 	//
 	public class XSPApplicationHost : BaseApplicationHost
 	{
-		public void ProcessRequest (int reqId, string localEPAddr, int localEPPort, string remoteEPAdds,
+		public void ProcessRequest (int reqId, long localEPAddr, int localEPPort, long remoteEPAdds,
 					int remoteEPPort, string verb, string path, string pathInfo,
 					string queryString, string protocol, byte [] inputBuffer, string redirect)
 		{
 			XSPRequestBroker broker = (XSPRequestBroker) RequestBroker;
-			IPEndPoint localEP = new IPEndPoint (IPAddress.Parse (localEPAddr), localEPPort);
-			IPEndPoint remoteEP = new IPEndPoint (IPAddress.Parse (remoteEPAdds), remoteEPPort);
+			IPEndPoint localEP = new IPEndPoint (localEPAddr, localEPPort);
+			IPEndPoint remoteEP = new IPEndPoint (remoteEPAdds, remoteEPPort);
 			XSPWorkerRequest mwr = new XSPWorkerRequest (reqId, broker, this, localEP, remoteEP, verb, path,
 								pathInfo, queryString, protocol, inputBuffer);
 
+			string translated = mwr.GetFilePathTranslated ();
+			if (path [path.Length - 1] != '/' && Directory.Exists (translated))
+				redirect = path + '/';
+
 			if (redirect != null) {
 				Redirect (mwr, redirect);
+				broker.UnregisterRequest (reqId);
 				return;
 			}
 
@@ -159,13 +197,14 @@ namespace Mono.ASPNET
 	{
 		ApplicationServer server;
 		LingeringNetworkStream stream;
-		RequestData rdata;
 		IPEndPoint remoteEP;
 		IPEndPoint localEP;
+		Socket sock;
 
 		public XSPWorker (Socket client, EndPoint localEP, ApplicationServer server)
 		{
-			stream = new LingeringNetworkStream (client, true);
+			stream = new LingeringNetworkStream (client, false);
+			sock = client;
 			this.server = server;
 
 			try {
@@ -174,6 +213,11 @@ namespace Mono.ASPNET
 			this.localEP = (IPEndPoint) localEP;
 		}
 
+		public int GetReuseCount ()
+		{
+			return server.GetAvailableReuses (sock);
+		}
+
 		public void Run (object state)
 		{
 			int requestId = -1;
@@ -196,8 +240,8 @@ namespace Mono.ASPNET
 
 				if (host == null) {
 					byte [] nf = HttpErrors.NotFound (rdata.Path);
-					stream.Write (nf, 0, nf.Length);
-					stream.Close ();
+					Write (nf, 0, nf.Length);
+					Close ();
 					return;
 				}
 				
@@ -206,19 +250,25 @@ namespace Mono.ASPNET
 				
 				string redirect;
 				vapp.Redirect (rdata.Path, out redirect);
-				host.ProcessRequest (requestId, localEP.Address.ToString(), localEP.Port,
-						remoteEP.Address.ToString(), remoteEP.Port, rdata.Verb,
+				host.ProcessRequest (requestId, localEP.Address.Address, localEP.Port,
+						remoteEP.Address.Address, remoteEP.Port, rdata.Verb,
 						rdata.Path, rdata.PathInfo, rdata.QueryString,
 						rdata.Protocol, rdata.InputBuffer, redirect);
 			} catch (Exception e) {
-				Console.WriteLine (e);
+				if (!(e is RequestLineException))
+					Console.WriteLine (e);
+
 				try {
-					byte [] error = HttpErrors.ServerError ();
-					stream.Write (error, 0, error.Length);
-					stream.Close ();
+					if (!(e is IOException)) {
+						byte [] error = HttpErrors.ServerError ();
+						Write (error, 0, error.Length);
+					}
 				} catch {}
-			} finally {
-				if (requestId != -1)
+				try {
+					Close ();
+				} catch {}
+
+				if (broker != null && requestId != -1)
 					broker.UnregisterRequest (requestId);
 			}
 		}
@@ -235,9 +285,23 @@ namespace Mono.ASPNET
 		
 		public void Close ()
 		{
-			stream.Close ();
+			Close (false);
 		}
-		
+
+		public void Close (bool keepAlive)
+		{
+			if (!keepAlive) {
+				stream.Close ();
+				sock.Close ();
+				server.RemoveSocket (sock);
+				return;
+			}
+
+			stream.EnableLingering = false;
+			stream.Close ();
+			server.ReuseSocket (sock);
+		}
+
 		public bool IsConnected ()
 		{
 			return stream.Connected;
