@@ -6,7 +6,28 @@
 //	Simon Waite (simon@psionics.demon.co.uk)
 //
 // (C) 2002,2003 Ximian, Inc (http://www.ximian.com)
+// (C) Copyright 2004 Novell, Inc. (http://www.novell.com)
 //
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections;
 using System.Configuration;
@@ -24,7 +45,6 @@ namespace Mono.ASPNET
 {
 	public class XSPWorkerRequest : MonoWorkerRequest
 	{
-		IApplicationHost appHost;
 		string verb;
 		string path;
 		string pathInfo;
@@ -39,20 +59,16 @@ namespace Mono.ASPNET
 		byte [] inputBuffer;
 		int inputLength;
 		int position;
-		EndPoint localEP;
 		EndPoint remoteEP;
 		bool sentConnection;
 		int localPort;
 		string localAddress;
 		int requestId;
 		XSPRequestBroker requestBroker;
+		bool keepAlive;
 		
-		static byte [] error500;
-
 		static string serverHeader;
 
-		static string dirSeparatorString = Path.DirectorySeparatorChar.ToString ();
-
 		static string [] indexFiles = { "index.aspx",
 						"Default.aspx",
 						"default.aspx",
@@ -78,14 +94,6 @@ namespace Mono.ASPNET
 
 			string indexes = ConfigurationSettings.AppSettings ["MonoServerDefaultIndexFiles"];
 			SetDefaultIndexFiles (indexes);
-
-			string s = "HTTP/1.0 500 Server error\r\n" +
-				   serverHeader + 
-				    "<html><body><h1>500 Server error</h1>\r\n" +
-				   "Your client sent a request that was not understood by this server.\r\n" +
-				   "</body></html>\r\n";
-			
-			error500 = Encoding.Default.GetBytes (s);
 		}
 
 		static void SetDefaultIndexFiles (string list)
@@ -105,7 +113,26 @@ namespace Mono.ASPNET
 
 			indexFiles = (string []) files.ToArray (typeof (string));
 		}
-
+		
+		static Stack bufferStack = new Stack ();
+		
+		static MemoryStream AllocateMemoryStream ()
+		{
+			lock (bufferStack) {
+				if (bufferStack.Count != 0)
+					return (MemoryStream) bufferStack.Pop ();
+			}
+			return new MemoryStream ();
+		}
+		
+		static void FreeMemoryStream (MemoryStream buf)
+		{
+			buf.SetLength (0);
+			lock (bufferStack) {
+				bufferStack.Push (buf);
+			}
+		}
+		
 		public XSPWorkerRequest (int requestId, 
 								XSPRequestBroker requestBroker, 
 								IApplicationHost appHost, 
@@ -121,15 +148,15 @@ namespace Mono.ASPNET
 		{
 			this.requestId = requestId;
 			this.requestBroker = requestBroker;
-			this.appHost = appHost;
-			this.localEP = localEP;
 			this.remoteEP = remoteEP;
 			this.verb = verb;
 			this.path = path;
 			this.pathInfo = pathInfo;
 			this.protocol = protocol;
-			if (protocol == "HTTP/1.1")
-				protocol = "HTTP/1.0";	// Only 1.0 supported by xsp standalone.
+			if (protocol == "HTTP/1.1") {
+				this.protocol = "HTTP/1.0";	// Only 1.0 supported by xsp standalone.
+				keepAlive = true;
+			}
 
 			this.queryString = queryString;
 			this.inputBuffer = inputBuffer;
@@ -137,16 +164,26 @@ namespace Mono.ASPNET
 			position = 0;
 
 			GetRequestHeaders ();
+			string cncHeader = (string) headers ["Connection"];
+			if (cncHeader != null) {
+				cncHeader = cncHeader.ToLower ();
+				if (cncHeader.IndexOf ("keep-alive") != -1)
+					keepAlive = true;
+
+				if (cncHeader.IndexOf ("close") != -1)
+					keepAlive = false;
+			}
+
 			responseHeaders = new StringBuilder ();
-			response = new MemoryStream ();
+			responseHeaders.Append (serverHeader);
+			response = AllocateMemoryStream ();
 			status = "HTTP/1.0 200 OK\r\n";
 			
 			localPort = ((IPEndPoint) localEP).Port;
 			localAddress = ((IPEndPoint) localEP).Address.ToString();
 		}
 		
-		public int RequestId
-		{
+		public override int RequestId {
 			get { return requestId; }
 		}
 
@@ -198,6 +235,8 @@ namespace Mono.ASPNET
 					
 
 				text.Append ((char) c);
+				if (text.Length > 8192)
+					throw new InvalidOperationException ("Line too long.");
 			}
 
 			return text.ToString ();
@@ -207,7 +246,8 @@ namespace Mono.ASPNET
 		{
 			try {
 				string line;
-				headers = new Hashtable ();
+				headers = new Hashtable (CaseInsensitiveHashCodeProvider.Default,
+							CaseInsensitiveComparer.Default);
 				while ((line = ReadLine ()) != null && line.Length > 0) {
 					int colon = line.IndexOf (':');
 					if (colon == -1 || line.Length < colon + 2)
@@ -216,8 +256,10 @@ namespace Mono.ASPNET
 					string value = line.Substring (colon + 1).Trim ();
 					headers [key] = value;
 				}
+			} catch (IOException ioe) {
+				throw;
 			} catch (Exception e) {
-				throw new Exception ("Error reading headers.");
+				throw new Exception ("Error reading headers.", e);
 			}
 		}
 
@@ -225,22 +267,57 @@ namespace Mono.ASPNET
 		{
 			WebTrace.WriteLine ("CloseConnection()");
 			if (requestBroker != null) {
-				requestBroker.Close (requestId);
+				requestBroker.Close (requestId, keepAlive);
 				requestBroker = null;
+				FreeMemoryStream (response);
+				response = null;
 			}
 		}
 
+		void AddConnectionHeader ()
+		{
+			if (!keepAlive) {
+				responseHeaders.Append ("Connection: close\r\n");
+				return;
+			}
+
+			int allowed = requestBroker.GetReuseCount (requestId);
+			if (allowed <= 0) {
+				keepAlive = false;
+				responseHeaders.Append ("Connection: close\r\n");
+				return;
+			}
+
+			responseHeaders.Append ("Keep-Alive: timeout=15, max=");
+			responseHeaders.Append (allowed.ToString ());
+			responseHeaders.Append ("\r\n");
+			responseHeaders.Append ("Connection: Keep-Alive\r\n");
+		}
+
 		public override void FlushResponse (bool finalFlush)
 		{
 			try {
 				if (!headersSent) {
-					responseHeaders.Insert (0, serverHeader);
 					responseHeaders.Insert (0, status);
 					if (!sentConnection)
-						responseHeaders.Append ("Connection: close\r\n");
+						AddConnectionHeader ();
 
 					responseHeaders.Append ("\r\n");
-					WriteString (responseHeaders.ToString ());
+					byte [] headerBytes = Encoding.GetBytes (responseHeaders.ToString ());
+					int oldLength = (int) response.Length;
+					if (oldLength == 0 || oldLength >= 32768) {
+						requestBroker.Write (requestId, headerBytes, 0, headerBytes.Length);
+					} else {
+						// Attempt not to send a minimum of 2 packets
+						int newLength = oldLength + headerBytes.Length;
+						response.SetLength (newLength);
+						byte [] buf = response.GetBuffer ();
+						Buffer.BlockCopy (buf, 0, buf, headerBytes.Length, oldLength);
+						Buffer.BlockCopy (headerBytes, 0, buf, 0, headerBytes.Length);
+						requestBroker.Write (requestId, buf, 0, newLength);
+						response.SetLength (0);
+					}
+
 					headersSent = true;
 				}
 
@@ -249,10 +326,12 @@ namespace Mono.ASPNET
 					requestBroker.Write (requestId, bytes, 0, (int) response.Length);
 				}
 				
-				requestBroker.Flush (requestId);
-				response.SetLength (0);
 				if (finalFlush)
 					CloseConnection ();
+				else {
+					requestBroker.Flush (requestId);
+					response.SetLength (0);
+				}
 			} catch (Exception e) {
 				WebTrace.WriteLine (e.ToString ());
 			}
@@ -466,13 +545,6 @@ namespace Mono.ASPNET
 			return true;
 		}
 
-		void WriteString (string s)
-		{
-			byte [] b = Encoding.GetBytes (s);
-			requestBroker.Write (requestId, b, 0, b.Length);
-
-		}
-
 		protected override bool GetRequestData ()
 		{
 			return TryDirectory ();
@@ -529,17 +601,24 @@ namespace Mono.ASPNET
 		public override void SendStatus (int statusCode, string statusDescription)
 		{
 			status = String.Format ("HTTP/1.0 {0} {1}\r\n", statusCode, statusDescription);
-			WebTrace.WriteLine ("SendStatus() -> " + status);
 		}
 
 		public override void SendUnknownResponseHeader (string name, string value)
 		{
 			WebTrace.WriteLine ("SendUnknownResponseHeader (" + name + ", " + value + ")");
-			if (String.Compare (name, "connection", true) == 0)
+			if (String.Compare (name, "connection", true) == 0) {
 				sentConnection = true;
+				if (value.ToLower ().IndexOf ("keep-alive") == -1) {
+					keepAlive = false;
+				}
+			}
 
-			if (!headersSent)
-				responseHeaders.AppendFormat ("{0}: {1}\r\n", name, value);
+			if (!headersSent) {
+				responseHeaders.Append (name);
+				responseHeaders.Append (": ");
+				responseHeaders.Append (value);
+				responseHeaders.Append ("\r\n");
+			}
 		}
 	}
 }
