@@ -1,11 +1,30 @@
 //
-// Web Server that uses ASP.NET hosting
+// server.cs: Web Server that uses ASP.NET hosting
 //
 // Authors:
 //	Gonzalo Paniagua Javier (gonzalo@ximian.com)
 //
 // (C) 2002,2003 Ximian, Inc (http://www.ximian.com)
-// (c) Copyright 2004 Novell, Inc. (http://www.novell.com)
+// (C) Copyright 2004 Novell, Inc. (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
 using System;
@@ -25,7 +44,7 @@ namespace Mono.ASPNET
 			Assembly assembly = Assembly.GetExecutingAssembly ();
 			string version = assembly.GetName ().Version.ToString ();
 			object [] att = assembly.GetCustomAttributes (typeof (AssemblyTitleAttribute), false);
-			string title = ((AssemblyTitleAttribute) att [0]).Title;
+			//string title = ((AssemblyTitleAttribute) att [0]).Title;
 			att = assembly.GetCustomAttributes (typeof (AssemblyCopyrightAttribute), false);
 			string copyright = ((AssemblyCopyrightAttribute) att [0]).Copyright;
 			att = assembly.GetCustomAttributes (typeof (AssemblyDescriptionAttribute), false);
@@ -39,28 +58,35 @@ namespace Mono.ASPNET
 #if MODMONO_SERVER
 			Console.WriteLine ("mod-mono-server.exe is a ASP.NET server used from mod_mono.");
 			Console.WriteLine ("Usage is:\n");
-			Console.WriteLine ("    mod-mono-server.exe [--root rootdir] [--applications APPS] [--filename file]");
-			Console.WriteLine ("            [--appconfigdir DIR] [--appconfigfile FILE]");
+			Console.WriteLine ("    mod-mono-server.exe [...]");
 			Console.WriteLine ();
-			Console.WriteLine ("    --filename file: the unix socket file name.");
+			Console.WriteLine ("    The arguments --filename and --port are mutually exlusive.");
+			Console.WriteLine ("    --filename file: a unix socket filename to listen on.");
 			Console.WriteLine ("                    Default value: /tmp/mod_mono_server");
 			Console.WriteLine ("                    AppSettings key name: MonoUnixSocket");
+			Console.WriteLine ();
 #else
 			Console.WriteLine ("XSP server is a sample server that hosts the ASP.NET runtime in a");
 			Console.WriteLine ("minimalistic HTTP server\n");
 			Console.WriteLine ("Usage is:\n");
-			Console.WriteLine ("    xsp.exe [--root rootdir] [--applications APPS]");
-			Console.WriteLine ("            [--appconfigdir DIR] [--appconfigfile FILE]");
-			Console.WriteLine ("            [--port N] [--address addr]");
+			Console.WriteLine ("    xsp.exe [...]");
 			Console.WriteLine ();
+#endif
 			Console.WriteLine ("    --port N: n is the tcp port to listen on.");
+#if MODMONO_SERVER
+			Console.WriteLine ("                    Default value: none");
+#else
 			Console.WriteLine ("                    Default value: 8080");
+#endif
 			Console.WriteLine ("                    AppSettings key name: MonoServerPort");
 			Console.WriteLine ();
 			Console.WriteLine ("    --address addr: addr is the ip address to listen on.");
+#if MODMONO_SERVER
+			Console.WriteLine ("                    Default value: 127.0.0.1");
+#else
 			Console.WriteLine ("                    Default value: 0.0.0.0");
-			Console.WriteLine ("                    AppSettings key name: MonoServerAddress");
 #endif
+			Console.WriteLine ("                    AppSettings key name: MonoServerAddress");
 			Console.WriteLine ();
 			Console.WriteLine ("    --root rootdir: the server changes to this directory before");
 			Console.WriteLine ("                    anything else.");
@@ -100,6 +126,11 @@ namespace Mono.ASPNET
 			Console.WriteLine ("                    Default value: /:.");
 			Console.WriteLine ("                    AppSettings key name: MonoApplications");
 			Console.WriteLine ();
+#if MODMONO_SERVER
+			Console.WriteLine ("    --terminate: gracefully terminates a running mod-mono-server instance.");
+			Console.WriteLine ("                 All other options but --filename or --address and --port");
+			Console.WriteLine ("                 are ignored if this option is provided.");
+#endif
 			Console.WriteLine ("    --nonstop: don't stop the server by pressing enter. Must be used");
 			Console.WriteLine ("               when the server has no controlling terminal.");
 			Console.WriteLine ();
@@ -119,7 +150,8 @@ namespace Mono.ASPNET
 			Root = 1 << 5,
 			FileName = 1 << 6,
 			Address = 1 << 7,
-			Port = 1 << 8
+			Port = 1 << 8,
+			Terminate = 1 << 9,
 		}
 
 		static void CheckAndSetOptions (string name, Options value, ref Options options)
@@ -132,23 +164,29 @@ namespace Mono.ASPNET
 			}
 
 			options |= value;
+			if ((options & Options.FileName) != 0 &&
+			    ((options & Options.Port) != 0 || (options & Options.Address) != 0)) {
+				ShowHelp ();
+				Console.WriteLine ();
+				Console.WriteLine ("ERROR: --port/--address and --filename are mutually exclusive");
+				Environment.Exit (1);
+			}
 		}
 		
 		public static int Main (string [] args)
 		{
 			bool nonstop = false;
-			bool verbose = true;
+			bool verbose = false;
 			Trace.Listeners.Add (new TextWriterTraceListener (Console.Out));
 			string apps = ConfigurationSettings.AppSettings ["MonoApplications"];
 			string appConfigDir = ConfigurationSettings.AppSettings ["MonoApplicationsConfigDir"];
 			string appConfigFile = ConfigurationSettings.AppSettings ["MonoApplicationsConfigFile"];
 			string rootDir = ConfigurationSettings.AppSettings ["MonoServerRootDir"];
-#if MODMONO_SERVER
-			string filename = ConfigurationSettings.AppSettings ["MonoUnixSocket"];
-#else
 			object oport;
 			string ip = ConfigurationSettings.AppSettings ["MonoServerAddress"];
-			
+#if MODMONO_SERVER
+			string filename = ConfigurationSettings.AppSettings ["MonoUnixSocket"];
+#endif
 			if (ip == "" || ip == null)
 				ip = "0.0.0.0";
 
@@ -156,7 +194,6 @@ namespace Mono.ASPNET
 			if (oport == null)
 				oport = 8080;
 
-#endif
 			Options options = 0;
 			for (int i = 0; i < args.Length; i++){
 				string a = args [i];
@@ -167,7 +204,10 @@ namespace Mono.ASPNET
 					CheckAndSetOptions (a, Options.FileName, ref options);
 					filename = args [++i];
 					break;
-#else
+				case "--terminate":
+					CheckAndSetOptions (a, Options.Terminate, ref options);
+					break;
+#endif
 				case "--port":
 					CheckAndSetOptions (a, Options.Port, ref options);
 					oport = args [++i];
@@ -176,7 +216,6 @@ namespace Mono.ASPNET
 					CheckAndSetOptions (a, Options.Address, ref options);
 					ip = args [++i];
 					break;
-#endif
 				case "--root":
 					CheckAndSetOptions (a, Options.Root, ref options);
 					rootDir = args [++i];
@@ -213,9 +252,20 @@ namespace Mono.ASPNET
 			}
 
 #if MODMONO_SERVER
-			if (filename == null || filename == "")
-				filename = "/tmp/mod_mono_server";
-#else
+			bool useTCP = ((options & Options.Port) != 0);
+			if (!useTCP) {
+				if (filename == null || filename == "")
+					filename = "/tmp/mod_mono_server";
+
+				if ((options & Options.Address) != 0) {
+					ShowHelp ();
+					Console.WriteLine ();
+					Console.WriteLine ("ERROR: --address without --port");
+					Environment.Exit (1);
+				}
+			}
+#endif
+			IPAddress ipaddr = null;
 			ushort port;
 			try {
 				port = Convert.ToUInt16 (oport);
@@ -225,12 +275,12 @@ namespace Mono.ASPNET
 			}
 
 			try {
-				IPAddress.Parse (ip);
+				ipaddr = IPAddress.Parse (ip);
 			} catch (Exception) {
 				Console.WriteLine ("The value given for the address is not valid: " + ip);
 				return 1;
 			}
-#endif
+
 			if (rootDir != null && rootDir != "") {
 				try {
 					Environment.CurrentDirectory = rootDir;
@@ -242,15 +292,33 @@ namespace Mono.ASPNET
 
 			rootDir = Directory.GetCurrentDirectory ();
 			
+			IWebSource webSource;
 #if MODMONO_SERVER
-			ModMonoWebSource webSource = new ModMonoWebSource ();
+			if (useTCP) {
+				webSource = new ModMonoTCPWebSource (ipaddr, port);
+			} else {
+				webSource = new ModMonoWebSource (filename);
+			}
+
+			if ((options & Options.Terminate) != 0) {
+				if (verbose)
+					Console.WriteLine ("Shutting down running mod-mono-server...");
+				
+				bool res = ((ModMonoWebSource) webSource).GracefulShutdown ();
+				if (verbose)
+					Console.WriteLine (res ? "Done." : "Failed");
+
+				return (res) ? 0 : 1;
+			}
+
 			ApplicationServer server = new ApplicationServer (webSource);
 #else
-			XSPWebSource webSource = new XSPWebSource ();
+			webSource = new XSPWebSource (ipaddr, port);
 			ApplicationServer server = new ApplicationServer (webSource);
 #endif
 			server.Verbose = verbose;
 
+			Console.WriteLine (Assembly.GetExecutingAssembly ().GetName ().Name);
 			if (apps != null)
 				server.AddApplicationsFromCommandLine (apps);
 
@@ -263,13 +331,14 @@ namespace Mono.ASPNET
 			if (apps == null && appConfigDir == null && appConfigFile == null)
 				server.AddApplicationsFromCommandLine ("/:.");
 #if MODMONO_SERVER
-			webSource.SetListenFile (filename);
-			Console.WriteLine ("Listening on: {0}", filename);
-#else
-			webSource.SetListenAddress (IPAddress.Parse (ip), port);
-			Console.WriteLine ("Listening on port: {0}", port);
-			Console.WriteLine ("Listening on address: {0}", ip);
+			if (!useTCP) {
+				Console.WriteLine ("Listening on: {0}", filename);
+			} else
 #endif
+			{
+				Console.WriteLine ("Listening on port: {0}", port);
+				Console.WriteLine ("Listening on address: {0}", ip);
+			}
 			
 			Console.WriteLine ("Root directory: {0}", rootDir);
 
@@ -280,6 +349,7 @@ namespace Mono.ASPNET
 				if (!nonstop) {
 					Console.WriteLine ("Hit Return to stop the server.");
 					Console.ReadLine ();
+					server.Stop ();
 				}
 			} catch (Exception e) {
 				Console.WriteLine ("Error: {0}", e.Message);
